require 'stripe'

module AT
  module Stripe
    module Customer
      REQUIRED_ACCESSORS = %w{email email= stripe_id stripe_id=}

      def self.included(base)
        self.enforce_attributes(base)
      end



      def create_customer(options={})
        options[:email] ||= self.email
        if @customer = ::Stripe::Customer.create(options)
          (self.stripe_id = @customer.id) and @customer
        end
      end

      def retrieve_customer
        @customer ||= ::Stripe::Customer.retrieve(self.stripe_id)
      end
      alias_method :customer, :retrieve_customer



      def create_card(card_token)
        customer.sources.create(source: card_token)
      end

      def retrieve_card(card_token=nil)
        card_token ||= unless (card = list_cards.data.first).nil?
          card.id
        end
        return if card_token.nil?
        customer.sources.retrieve(card_token)
      end

      def purge_cards
        list_cards.map(&:delete)
      end

      def list_cards(limit=100)
        customer.sources.all(object: 'card', limit: limit)
      end



      def create_charge(options)
        unless options.key?(:source) || options.key?(:customer)
          options[:customer] = self.stripe_id
        end
        ::Stripe::Charge.create(options)
      end

      def list_charges(limit=100)
        ::Stripe::Charge.all(customer: customer, limit: limit)
      end



      private

      def self.enforce_attributes(base)
        lookup = self.method_lookup(base)
        REQUIRED_ACCESSORS.each do |attribute|
          unless base.send(lookup, attribute)
            raise "receiving class must have \##{attribute}"
          end
        end
      end

      def self.method_lookup(base)
        # more stupid special cases coming soon
        key = base.superclass.to_s
        {
          'ActiveRecord::Base' => :attribute_method?
        }.fetch(key, :public_method_defined?)
      end
    end
  end
end
